@@ -27,10 +27,10 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)admin.c	1.62 11/09/03 J. Schilling
+ * @(#)admin.c	1.66 11/09/15 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)admin.c 1.62 11/09/03 J. Schilling"
+#pragma ident "@(#)admin.c 1.66 11/09/15 J. Schilling"
 #endif
 /*
  * @(#)admin.c 1.39 06/12/12
@@ -116,6 +116,7 @@ static char	stdin_file_buf [20];
 static char	*ifile, *tfile;
 static char	*dir_name;
 static time_t	ifile_mtime;
+static int	Ncomma = 0;
 static int	Nsdot = 1;
 static int	Nsubd;
 static char	*Nprefix;
@@ -139,7 +140,7 @@ static int	asub, esub;
 static int	check_id;
 static struct	utsname	un;
 static char	*uuname;
-static int 	Encoded = 0;
+static int 	Encoded = EF_TEXT;	/* Default encoding is '0' */
 static off_t	Encodeflag_offset;	/* offset in file where encoded flag is stored */
 static off_t	Checksum_offset;	/* offset in file where g-file hash is stored */
 
@@ -566,6 +567,10 @@ char *argv[];
 
 	if (HADUCN) {
 		HADI = HADN = 1;
+		if (*Nparm == ',') {
+			Ncomma = 1;
+			Nparm++;
+		}
 		Nsdot = sccsfile(Nparm);
 		if (Nsdot) {				/* Nparm ends in s. */
 			if (strlen(sname(Nparm)) > 2)
@@ -996,7 +1001,7 @@ char	*afile;
 	store them. Check to see if the flag read is one that
 	should be deleted.
 	*/
-	if (!HADN)
+	if (!HADN) {
 		while (((cp = getline(&gpkt)) != NULL) &&
 				(*cp++ == CTLCHAR && *cp++ == FLAG)) {
 
@@ -1041,6 +1046,14 @@ char	*afile;
 					while (*q)	/* find and */
 						q++;	/* zero newline */
 					*--q = '\0';	/* character */
+					if (k == ENCODEFLAG - 'a') {
+						int	i;
+
+						NONBLANK(cp);
+						cp = satoi(cp, &i);
+						if (*cp == '\n')
+							gpkt.p_encoding = i;
+					}
 				}
 			}
 			if (rm_flag[k]) {
@@ -1056,7 +1069,7 @@ char	*afile;
 				else had_flag[k] = 0;
 			}
 		}
-
+	}
 
 	/*
 	Write out flags.
@@ -1126,8 +1139,8 @@ char	*afile;
 			by setting the value to 1.
 		 	*/
 			Encodeflag_offset = ftell(Xiop);
-			sprintf(line,"%c%c %c 0\n",
-				CTLCHAR,FLAG,ENCODEFLAG);
+			sprintf(line,"%c%c %c %d\n",
+				CTLCHAR, FLAG, ENCODEFLAG, Encoded);
 			putline(&gpkt,line);
 		}
 		/*
@@ -1183,7 +1196,7 @@ char	*afile;
 		putline(&gpkt,line);
 
 		if (HADB)
-			Encoded=1;
+			Encoded |= EF_UUENCODE;
 		if (HADI) {		/* get body */
 
 			/*
@@ -1273,14 +1286,14 @@ char	*afile;
 			if (!HADB) {
 			   stats.s_ins = fgetchk(iptr, ifile, &gpkt, 1);
 			   if (stats.s_ins == -1 ) {
-			      Encoded = 1;
+				Encoded |= EF_UUENCODE;
 			   } else {
-			      Encoded = 0;
+				Encoded &= ~EF_UUENCODE;	/* Keep EF_GZIP */
 			   }
 			} else {
-			   Encoded = 1;
+				Encoded |= EF_UUENCODE;
 			}
-			if (Encoded) {
+			if (Encoded & EF_UUENCODE) {
 			   /* non-ascii characters in file, encode them */
 			   iptr = code(iptr, afile, offset, thash, &gpkt);
 			   stats.s_ins = fgetchk(iptr, ifile, &gpkt, 0);
@@ -1341,7 +1354,7 @@ char	*afile;
 	   the hash value
 	 */
 	
-	if (Encoded)
+	if (Encoded & EF_UUENCODE)
 	{
 		strcpy(line,"0");
 		q = (signed char *) line;
@@ -1352,8 +1365,8 @@ char	*afile;
 		while (*q)
 			gpkt.p_nhash += *q++;
 		fseek(Xiop, Encodeflag_offset, SEEK_SET);
-		fprintf(Xiop,"%c%c %c 1\n",
-			CTLCHAR, FLAG, ENCODEFLAG);
+		fprintf(Xiop,"%c%c %c %d\n",
+			CTLCHAR, FLAG, ENCODEFLAG, Encoded);
 	}
 
 	/*
@@ -1375,6 +1388,18 @@ char	*afile;
 			chmod(gpkt.p_file, sbuf.st_mode);
 			chown(gpkt.p_file,sbuf.st_uid, sbuf.st_gid);
 		}
+		if (HADI && *ifile && Ncomma) {
+			char cfile[FILESIZE];
+			register char *snp;
+
+			snp = sname(ifile);
+
+			strlcpy(cfile, ifile, sizeof (cfile));
+			cfile[snp-ifile] = '\0';
+			strlcat(cfile, ",", sizeof (cfile));
+			if (strlcat(cfile, snp, sizeof (cfile)) < sizeof (cfile))
+				rename(ifile, cfile);
+		}
 		xrm();
 		uname(&un);
 		uuname = un.nodename;
@@ -1490,7 +1515,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 		putline(pkt, lastline);
 
 		if (check_id && Did_id == 0) {
-			(void)chkid(line, flag_p['i'-'a'], flag_p);
+			(void)chkid(line, flag_p[IDFLAG - 'a'], flag_p);
 		}
 		ibase += idx;
 	}
@@ -1539,7 +1564,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 	      }
 	   }   
 	   if (check_id && Did_id == 0) {
-	      (void)chkid(line, flag_p['i'-'a'], flag_p);
+	      (void)chkid(line, flag_p[IDFLAG - 'a'], flag_p);
 	   }
 	   putline(pkt, line);
 	   (void)memset(line, '\377', sizeof (line));
