@@ -27,10 +27,10 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)delta.c	1.45 11/09/03 J. Schilling
+ * @(#)delta.c	1.47 11/09/15 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)delta.c 1.45 11/09/03 J. Schilling"
+#pragma ident "@(#)delta.c 1.47 11/09/15 J. Schilling"
 #endif
 /*
  * @(#)delta.c 1.40 06/12/12
@@ -418,7 +418,7 @@ char *file;
 	/* if encode flag is set, encode the g-file before diffing it
 	 * with the s.file
 	 */
-	if (Sflags[ENCODEFLAG - 'a'] && (strcmp(Sflags[ENCODEFLAG - 'a'],"1") == 0)) {
+	if (gpkt.p_encoding & EF_UUENCODE) {
 		efp = xfcreat(auxf(gpkt.p_file,'e'),0644);
 #ifdef	USE_SETVBUF
 		setvbuf(efp, NULL, _IOFBF, VBUF_SIZE);
@@ -462,8 +462,7 @@ char *file;
 	if (Sflags[EXPANDFLAG - 'a'] && *(Sflags[EXPANDFLAG - 'a']) == '\0') {
 		Did_id++;		/* No need to scan for keywds */
 	}
-	if ((Sflags[ENCODEFLAG - 'a'] == NULL) ||
-	    (strcmp(Sflags[ENCODEFLAG - 'a'],"0") == 0)) {
+	if ((gpkt.p_encoding & EF_UUENCODE) == 0) {
 		fgetchk(gfilename, &gpkt);
 		number_of_lines = gpkt.p_glines;
 	} else {
@@ -475,7 +474,7 @@ char *file;
  	   			number_of_lines++;
 			}
 			if (Did_id == 0) {
-				chkid(line,Sflags['i'-'a'], Sflags);
+				chkid(line,Sflags[IDFLAG - 'a'], Sflags);
 			}
 		}
 	}
@@ -540,7 +539,7 @@ char *file;
         	}
 		diffloop = 1;
 		flushto(&gpkt,EUSERTXT,0);
-		if (Sflags[ENCODEFLAG - 'a'] && (strcmp(Sflags[ENCODEFLAG - 'a'],"1") == 0))
+		if (gpkt.p_encoding & EF_UUENCODE)
 			Diffin = dodiff(auxf(gpkt.p_file,'e'),dfilename,difflim);
 		else
 			Diffin = dodiff(gfilename, dfilename, difflim);
@@ -626,8 +625,7 @@ command, to check the differences found between two files.
 			break;			/* exit while loop */
 		}
 	}
-	if (Sflags[ENCODEFLAG - 'a'] && 
-	    (strcmp(Sflags[ENCODEFLAG - 'a'],"1") == 0)) {
+	if (gpkt.p_encoding & EF_UUENCODE) {
 		unlink(auxf(gpkt.p_file,'e'));
 	}
 	unlink(dfilename);
@@ -1086,15 +1084,13 @@ int	linenum, n, ser;
  	char	str[BUFSIZ];
  	int 	first;
 	int	nonl = 0;
-	extern char *Sflags[];
 
 	/*
 	 * We only count newlines and thus need to add one to number_of_lines
 	 * to get the same view as bdiff/diff.
 	 */
 	if ((pkt->p_props & CK_NONL) && (linenum + n) >= (number_of_lines+1)) {
-		if ((Sflags[ENCODEFLAG - 'a'] == NULL) ||
-		    (strcmp(Sflags[ENCODEFLAG - 'a'],"0") == 0)) {
+		if ((gpkt.p_encoding & EF_UUENCODE) == 0) {
 			nonl++;
 		}
 	}
@@ -1387,7 +1383,7 @@ struct	packet	*pkt;
 		}
 		line[idx] = '\0';
 		if (Did_id == 0) {
-			chkid(line, Sflags['i'-'a'], Sflags);
+			chkid(line, Sflags[IDFLAG - 'a'], Sflags);
 		}
 	}
 #else	/* !RECORD_IO */
@@ -1427,7 +1423,7 @@ struct	packet	*pkt;
 	      }
 	   }   
 	   if (Did_id == 0) {
-		chkid(line, Sflags['i'-'a'], Sflags);
+		chkid(line, Sflags[IDFLAG - 'a'], Sflags);
 	   }
 	   (void)memset(line, '\377', sizeof (line));
 	}
