@@ -27,10 +27,10 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)get.c	1.45 11/09/03 J. Schilling
+ * @(#)get.c	1.47 11/09/15 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)get.c 1.45 11/09/03 J. Schilling"
+#pragma ident "@(#)get.c 1.47 11/09/15 J. Schilling"
 #endif
 /*
  * @(#)get.c 1.59 06/12/12
@@ -529,8 +529,7 @@ char *file;
 			}
 		}
 		gpkt.p_ghash = 0;		/* Reset ghash */
-		if (Sflags[ENCODEFLAG - 'a'] &&
-		    (strcmp(Sflags[ENCODEFLAG - 'a'],"1") == 0)) {
+		if (gpkt.p_encoding & EF_UUENCODE) {
 			while (readmod(&gpkt)) {
 				decode(gpkt.p_line,gpkt.p_gout);
 			}
@@ -989,8 +988,8 @@ char line[];
 	expand_XIDs = Sflags[EXTENSFLAG - 'a'] != NULL || list_expand_IDs != NULL;
 	tp = tline;
 	for (lp=line; *lp != 0; lp++) {
-		if ((!Did_id) && (Sflags['i'-'a']) &&
-		    (!(strncmp(Sflags['i'-'a'],lp,strlen(Sflags['i'-'a'])))))
+		if ((!Did_id) && (Sflags[IDFLAG - 'a']) &&
+		    (!(strncmp(Sflags[IDFLAG-'a'],lp,strlen(Sflags[IDFLAG-'a'])))))
 				++Did_id;
 		if (lp[0] == '%' && lp[1] != 0 && lp[2] == '%') {
 			expand_ID = ++lp;
@@ -1137,7 +1136,7 @@ char line[];
 				tp = trans(tp,str);
 				continue;
 			}
-			if (!(Sflags['i'-'a']))
+			if (!(Sflags[IDFLAG - 'a']))
 				++Did_id;
 			lp++;
 		} else if (strncmp(lp, "%sccs.include.", 14) == 0) {
