@@ -27,10 +27,10 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)prs.c	1.32 11/09/03 J. Schilling
+ * @(#)prs.c	1.33 11/09/15 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)prs.c 1.32 11/09/03 J. Schilling"
+#pragma ident "@(#)prs.c 1.33 11/09/15 J. Schilling"
 #endif
 /*
  * @(#)prs.c 1.33 06/12/12
@@ -1545,12 +1545,16 @@ printflags()
 	if ((k = (Sflags[DEFTFLAG - 'a'])) != NULL)  /* check for 'default SID' flag */
 		printf(gettext("default SID\t%s\n"), k);
 	if ((k = (Sflags[ENCODEFLAG - 'a'])) != NULL) {	/* check for 'encoded' flag */
+		int	i;
+
+		NONBLANK(k);
+		k = satoi(k, &i);
 		/*
 		 * The semantics of this flag are unusual: report its
 		 * existence only if an operand with value '1' is present.
 		 * Yes, the test below is sloppy...
 		 */
-		if (*k == '1')
+		if (*k == '\0' && (i & EF_UUENCODE))
 			printf(gettext("encoded\n"));
 	}
 	if ((k = (Sflags[FLORFLAG - 'a'])) != NULL)	/* check for 'floor' flag */
