@@ -27,10 +27,10 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)rmchg.c	1.28 11/07/04 J. Schilling
+ * @(#)rmchg.c	1.29 11/09/07 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)rmchg.c 1.28 11/07/04 J. Schilling"
+#pragma ident "@(#)rmchg.c 1.29 11/09/07 J. Schilling"
 #endif
 /*
  * @(#)rmchg.c 1.19 06/12/12
@@ -173,7 +173,7 @@ char *argv[];
 			}
 			no_arg = 0;
 			i = current_optind;
-		        c = getopt(argc, argv, "-r:m:y:zqV(version)");
+		        c = getopt(argc, argv, "-r:m:y:dzqV(version)");
 
 				/* this takes care of options given after
 				** file names.
@@ -202,6 +202,8 @@ char *argv[];
 				Sidhold=p;
 				chksid(sid_ab(p,&sid),&sid);
 				break;
+			case 'd':	/* rmdel -d -> fully discard delta */
+				break;
 			case 'm':	/* MR entry */
 				Mrs = p;
 				repl(Mrs,'\n',' ');
@@ -228,7 +230,9 @@ char *argv[];
 				break;
 
 			case 'V':		/* version */
-				printf("rmchg %s-SCCS version %s %s (%s-%s-%s)\n",
+				p = sname(argv[0]);
+				printf("%s %s-SCCS version %s %s (%s-%s-%s)\n",
+					equal(p, "cdc") ? "cdc": "rmdel",
 					PROVIDER,
 					VERSION,
 					VDATE,
@@ -236,7 +240,13 @@ char *argv[];
 				exit(EX_OK);
 
 			default:
-				fatal(gettext("Usage: rmdel [ -r SID ] s.filename ..."));
+				p = sname(argv[0]);
+				if (equal(p,"cdc"))
+					fatal(gettext(
+					"Usage: cdc -r SID [-mmr-list] [-y [comment]] s.filename ..."));
+				else
+					fatal(gettext(
+					"Usage: rmdel -r SID s.filename ..."));
 			}
 
 			/*
@@ -262,12 +272,15 @@ char *argv[];
 
 	if (*(p = sname(argv[0])) == 'n')
 		p++;
-	if (equal(p,NOGETTEXT("rmdel")))
+	if (equal(p,NOGETTEXT("rmdel"))) {
 		D_type = 'R';		/* invoked as 'rmdel' */
-	else if (equal(p,"cdc"))
+		if (HADD)
+			D_type = '\0';	/* invoked as 'rmdel -d' */
+	} else if (equal(p,"cdc")) {
 		D_type = 'D';		/* invoked as 'cdc' */
-	else
+	} else {
 		fatal(gettext("bad invocation (rc10)"));
+	}
 	if (! logname())
 		fatal(gettext("User ID not in password file (cm9)"));
 
@@ -427,7 +440,7 @@ char *file;
 	'leaf' delta, and if ok,
 	process the body.
 	*/
-	if (D_type == 'R') {
+	if (D_type == 'R' || D_type == '\0') {
 		struct idel *ptr;
 		for (n = maxser(&gpkt); n > D_serial; n--) {
 			ptr = &gpkt.p_idel[n];
