@@ -28,12 +28,12 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)defines.h	1.51 11/09/02 J. Schilling
+ * @(#)defines.h	1.52 11/09/15 J. Schilling
  */
 #ifndef	_HDR_DEFINES_H
 #define	_HDR_DEFINES_H
 #if defined(sun)
-#pragma ident "@(#)defines.h 1.51 11/09/02 J. Schilling"
+#pragma ident "@(#)defines.h 1.52 11/09/15 J. Schilling"
 #endif
 /*
  * @(#)defines.h 1.21 06/12/12
@@ -387,6 +387,7 @@ struct packet {
 	int	p_slnno;	/* line number of current input line */
 	char	p_wrttn;	/* written flag (!0 = written) */
 	char	p_keep;		/* keep switch for readmod() */
+	char	p_encoding;	/* encoding flags for data */
 	uint16_t *p_hash;	/* ptr to array of checksums */
 	struct	apply *p_apply;	/* ptr to apply array */
 	struct	queue *p_q;	/* ptr to control queue */
@@ -394,6 +395,7 @@ struct packet {
 #ifndef	HAVE_SETVBUF
 	char	p_buf[BUFSIZ];	/* input file buffer */
 #endif
+	char	*p_linebase;	/* base addr of malloc()ed line */
 	char	*p_lineptr;	/* begin of line past escape process */
 	char	*p_line;	/* buffer for getline() */
 	size_t	p_line_length;	/* actual line length for getline() */
@@ -425,6 +427,16 @@ struct packet {
 #define	PF_V6TAGS	4	/* SCCS V6 features found in hist file	  */
 #define	PF_NONL		8	/* This line has no newline		  */
 
+/*
+ * Encoding flags (p_encoding)
+ *
+ * The uuencoding takes place before the interleaved delta is created,
+ * the gzip compression takes place on the interleaved delta block.
+ */
+#define	EF_TEXT		0	/* Pure text in interleaved delta	  */
+#define	EF_UUENCODE	1	/* UUencoded binary in interleaved delta  */
+#define	EF_GZIP		4	/* Gzipped interleaved delta		  */
+
 /*
  * Flags to collect reasons for non-Text files also used in p_props
  */
