@@ -27,10 +27,10 @@
 /*
  * This file contains modifications Copyright 2006-2011 J. Schilling
  *
- * @(#)dodelt.c	1.11 11/09/02 J. Schilling
+ * @(#)dodelt.c	1.13 11/09/14 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)dodelt.c 1.11 11/09/02 J. Schilling"
+#pragma ident "@(#)dodelt.c 1.13 11/09/14 J. Schilling"
 #endif
 /*
  * @(#)dodelt.c 1.8 06/12/12
@@ -62,10 +62,11 @@ struct stats *statp;
 struct sid *sidp;
 char type;
 {
-	char *c;
+	char *c = NULL;
 	struct deltab dt;
 	register struct idel *rdp = NULL;
 	int n, founddel;
+	int	lhash;
 	register char *p;
 	time_t	TN;
 
@@ -77,6 +78,7 @@ char type;
 	if (pkt->p_flags & PF_GMT)
 		TN += gmtoff(Timenow);
 	stats_ab(pkt,statp);
+	lhash = pkt->p_clhash;
 	while (getadel(pkt,&dt) == BDELTAB) {
 		if (pkt->p_idel == 0) {
 			if (TN < dt.d_dtime.dt_sec)
@@ -112,6 +114,28 @@ char type;
 					pkt->p_uclhash += type;
 					*p = type;
 				}
+				if (type == 0) {
+					extern	FILE	*Xiop;
+
+					/*
+					 * Go back before last stats line. The
+					 * value 21 is the lenght of any stats
+					 * line. Never change this length.
+					 */
+					pkt->p_nhash -= lhash;
+					fseek(Xiop, (off_t)-21, SEEK_CUR);
+					/*
+					 * Skip this delta table entry.
+					 */
+					pkt->p_wrttn = 1;
+					while ((c = getline(pkt)) != NULL) {
+						pkt->p_wrttn = 1;
+						if (pkt->p_line[0] != CTLCHAR)
+							break;
+						if (pkt->p_line[1] == EDELTAB)
+							break;
+					}
+				}
 			}
 			else
 				First_esc = founddel = First_cmt = 0;
@@ -123,6 +147,8 @@ char type;
 			rdp->i_sid.s_seq = dt.d_sid.s_seq;
 			rdp->i_pred = dt.d_pred;
 			rdp->i_datetime = dt.d_dtime.dt_sec;
+			if (founddel && type == 0)	/* Already skipped */
+				goto nextdelta;
 		}
 		while ((c = getline(pkt)) != NULL)
 			if (pkt->p_line[0] != CTLCHAR)
@@ -163,10 +189,12 @@ char type;
 				}
 				break;
 			}
+	nextdelta:
 		if (c == NULL || pkt->p_line[0] != CTLCHAR || getline(pkt) == NULL)
 			fmterr(pkt);
 		if (pkt->p_line[0] != CTLCHAR || pkt->p_line[1] != STATS)
 			break;
+		lhash = pkt->p_clhash;
 	}
 	return(pkt->p_idel);
 }
